"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Field = void 0;

var _bcryptjs = _interopRequireDefault(require("bcryptjs"));

var _utils = require("./utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

var Field = /*#__PURE__*/function () {
  /**
   * Constructor
   *
   * @param {Object} config
   */
  function Field() {
    var config = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    _classCallCheck(this, Field);

    // remove undefined configs to be overwrite by default config
    Object.keys(config).forEach(function (key) {
      config[key] === undefined && delete config[key];
    }); // create default configs

    config = Object.assign({
      type: null,
      isModel: false,
      isArray: false,
      required: false,
      valid: false,
      filter_relationship: false,
      target: false,
      "default": false,
      min_length: false,
      max_length: false,
      attributes: false,
      labels: [],
      set: function set(value) {
        return value;
      },
      get: function get(value) {
        return value;
      }
    }, config); // put config on this

    Object.assign(this, config);
  }
  /**
   * Return the String for Label to use on Cypher
   *
   * @return {String}
   */


  _createClass(Field, [{
    key: "getLabelName",
    value: function getLabelName() {
      return this.labels.join(':').toUpperCase();
    }
    /**
     * Check if has Default value and return
     *
     * @param {String} value
     */

  }, {
    key: "hasDefaultValue",
    value: function hasDefaultValue(value) {
      if (value === undefined && this["default"]) {
        return this["default"] instanceof Function ? this["default"]() : this["default"];
      }

      return false;
    }
    /**
     * Check validations for each Field
     *
     * @param {String} key
     * @param {String} value
     */

  }, {
    key: "checkValidation",
    value: function checkValidation(key, value) {
      // valid
      if (this.required && this.valid && !this.valid.includes(value) || value && this.valid && !this.valid.includes(value)) {
        throw new Error("{ \"key\": \"".concat(key, "\", \"msg\": \"Field is not a valid option ").concat(JSON.stringify(this.valid).replace(/"/g, "'"), "\" }"));
      } // max_length


      if (this.max_length && value && value.length > this.max_length) {
        throw new Error("{ \"key\": \"".concat(key, "\", \"msg\": \"Field has more than ").concat(this.max_length, " characters\" }"));
      } // min_length


      if (this.min_length && value && value.length < this.min_length) {
        throw new Error("{ \"key\": \"".concat(key, "\", \"msg\": \"Field has less than ").concat(this.min_length, " characters\" }"));
      } // required


      if (this.required && value === undefined) {
        throw new Error("{ \"key\": \"".concat(key, "\", \"msg\": \"Field is required\" }"));
      }

      return value;
    }
    /**
     * @static
     * Generate Field instance
     * for string attributes
     *
     * @return {Field}
     */

  }], [{
    key: "String",
    value: function String() {
      var obj = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      var field = new this({
        type: 'string',
        set: obj.set,
        get: obj.get,
        required: obj.required,
        min_length: obj.min_length,
        max_length: obj.max_length,
        "default": obj["default"],
        valid: obj.valid
      });
      return field;
    }
    /**
     * @static
     * Generate Field instance
     * for integer attributes
     *
     * @return {Field}
     */

  }, {
    key: "Integer",
    value: function Integer() {
      var obj = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      var field = new this({
        type: 'integer',
        set: obj.set,
        get: obj.get,
        required: obj.required,
        "default": obj["default"]
      });
      return field;
    }
    /**
     * @static
     * Generate Field instance
     * for float attributes
     *
     * @return {Field}
     */

  }, {
    key: "Float",
    value: function Float() {
      var obj = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      var field = new this({
        type: 'float',
        set: obj.set,
        get: obj.get,
        required: obj.required,
        "default": obj["default"]
      });
      return field;
    }
    /**
     * @static
     * Generate Field instance
     * for hash attributes, automatic md5 to save
     *
     * @return {Field}
     */

  }, {
    key: "Hash",
    value: function Hash() {
      var obj = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

      var salt = _bcryptjs["default"].genSaltSync(10);

      var field = new this({
        type: 'hash',
        required: obj.required,
        set: function set(value) {
          return _bcryptjs["default"].hashSync("".concat(value), salt);
        },
        checkHash: function checkHash(value, saved) {
          return _bcryptjs["default"].compareSync("".concat(value), "".concat(saved));
        }
      });
      return field;
    }
    /**
     * @static
     * Generate Field instance
     * for json attributes
     *
     * @return {Field}
     */

  }, {
    key: "JSON",
    value: function (_JSON) {
      function JSON() {
        return _JSON.apply(this, arguments);
      }

      JSON.toString = function () {
        return _JSON.toString();
      };

      return JSON;
    }(function () {
      var obj = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      var field = new this({
        type: 'json',
        required: obj.required,
        "default": obj["default"],
        set: function set(value) {
          return JSON.stringify(value);
        },
        get: function get(value) {
          return JSON.parse(value);
        }
      });
      return field;
    })
    /**
     * @static
     * Generate Field instance
     * for boolean attributes
     *
     * @return {Field}
     */

  }, {
    key: "Boolean",
    value: function Boolean() {
      var obj = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      var field = new this({
        type: 'boolean',
        "default": obj["default"],
        set: function set(value) {
          return value === true || value === 'true' ? 'true' : 'false';
        },
        get: function get(value) {
          return value === true || value === 'true' ? true : false;
        }
      });
      return field;
    }
    /**
     * @static
     * Generate Field instance
     * for datetime attributes
     *
     * @return {Field}
     */

  }, {
    key: "DateTime",
    value: function DateTime() {
      var obj = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      var field = new this({
        type: 'datetime',
        required: obj.required,
        "default": obj["default"],
        set: function set(objDate) {
          return objDate.toISOString();
        },
        get: function get(value) {
          return (0, _utils.parseISOString)(value);
        }
      });
      return field;
    }
    /**
     * @static
     * Generate Field instance
     * for date attributes
     *
     * @return {Field}
     */

  }, {
    key: "Date",
    value: function Date() {
      var obj = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      var field = new this({
        type: 'date',
        required: obj.required,
        "default": obj["default"]
      });
      return field;
    }
    /**
     * @static
     * Generate Field instance
     * for time attributes
     *
     * @return {Field}
     */

  }, {
    key: "Time",
    value: function Time() {
      var obj = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      var field = new this({
        type: 'time',
        required: obj.required,
        "default": obj["default"]
      });
      return field;
    }
    /**
     * @static
     * Generate Field instance
     * for relationship attributes
     *
     * @return {Field}
     */

  }, {
    key: "Relationship",
    value: function Relationship() {
      var obj = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      var field = new this({
        type: 'relationship',
        isModel: true,
        required: obj.required,
        target: obj.target,
        labels: obj.labels,
        filter_relationship: obj.filter_relationship,
        filter_node: obj.filter_node,
        attributes: obj.attributes
      });
      return field;
    }
    /**
     * @static
     * Generate Field instance
     * for relationships attributes
     *
     * @return {Field}
     */

  }, {
    key: "Relationships",
    value: function Relationships() {
      var obj = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      var field = new this({
        type: 'relationships',
        isModel: true,
        isArray: true,
        required: obj.required,
        target: obj.target,
        labels: obj.labels,
        filter_relationship: obj.filter_relationship,
        filter_node: obj.filter_node,
        attributes: obj.attributes
      });
      return field;
    }
  }]);

  return Field;
}();

exports.Field = Field;