"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.checkWith = exports.hydrate = void 0;

var _utils = require("./utils");

var _collection = require("./collection");

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(n); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { if (typeof Symbol === "undefined" || !(Symbol.iterator in Object(arr))) return; var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**
 * Hydrate the model with the values of database
 *
 * @param {Model} model
 * @param {JSON} record
 * @param {Boolean} isArray
 * @param {Integer} level
 * @param {Boolean} onlyRelation
 */
var hydrate = function hydrate(model, record, with_related) {
  var level = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 0;
  var relationFieldLookup = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : null;
  var previous = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : null;

  var data = _objectSpread({}, record._fields[record._fieldLookup[model.getAliasName()]], {}, record._fields[record._fieldLookup[relationFieldLookup]]); // empty data should return null


  if (Object.keys(data).length === 0) return null; // save node ID

  model._values.id = data.id;
  (0, _utils.createOnlyGetter)(model, 'id', _utils.convertID); // console.log('data', data)
  // the relations has attributes

  if (previous) for (var _i = 0, _Object$entries = Object.entries(previous.attributes); _i < _Object$entries.length; _i++) {
    var _Object$entries$_i = _slicedToArray(_Object$entries[_i], 2),
        relKey = _Object$entries$_i[0],
        relAttr = _Object$entries$_i[1];

    // create getter and setter for that attribute inside _values
    (0, _utils.createGetterAndSetter)(model, relKey, relAttr.set, relAttr.get); // if not array should be linked at _values directed

    if (data[relKey]) model._values[relKey] = data[relKey];
  } // hydrate node Fields

  for (var _i2 = 0, _Object$entries2 = Object.entries(model._attributes); _i2 < _Object$entries2.length; _i2++) {
    var _Object$entries2$_i = _slicedToArray(_Object$entries2[_i2], 2),
        key = _Object$entries2$_i[0],
        field = _Object$entries2$_i[1];

    if (field.isModel) {
      // the information should be requested to be hydrated
      var _checkWith = checkWith(level, key, with_related),
          _checkWith2 = _slicedToArray(_checkWith, 1),
          found_condition = _checkWith2[0];

      if (found_condition) {
        if (field.isArray) {
          // should rehydrate
          // should return as Array (Object with key)
          // if is array should create the key as array and push for each record
          var next = record._fields[record._fieldLookup[key]]; // create a empty Collection

          if (model._values[key] === undefined) {
            // create getter and setter for that attribute inside _values
            (0, _utils.createGetterAndSetter)(model, key, field.set, field.get);
            model._values[key] = new _collection.Collection(); // this is the result

            model._values["".concat(key, "_ids")] = [];
          }

          if (next === null || next === void 0 ? void 0 : next.id) {
            var id = (0, _utils.convertID)(next.id);
            var targetModel = void 0;

            if (!model._values["".concat(key, "_ids")].includes(id)) {
              model._values["".concat(key, "_ids")].push(id);

              targetModel = new field.target(undefined, model._state);
              targetModel._state = model._state;
            } else {
              targetModel = model._values[key][model._values["".concat(key, "_ids")].indexOf(id)];
            }

            targetModel._alias = key; // console.log('hidratando array', model.getAliasName(), key, model._values[`${key}_ids`].indexOf(id))

            model._values[key][model._values["".concat(key, "_ids")].indexOf(id)] = hydrate(targetModel, record, with_related, level + 1, "".concat(model.getAliasName(), "_").concat(key), field);
          }
        } else {
          // should return as Object
          // hydrate the model
          // console.log('hidratando object', model.getAliasName(), key, record)
          var _targetModel = model._values[key] || new field.target(undefined, model._state);

          _targetModel._state = model._state;
          _targetModel._alias = key;
          var hydrated = hydrate(_targetModel, record, with_related, level + 1, "".concat(model.getAliasName(), "_").concat(key), field); // create getter and setter for that attribute inside _values

          (0, _utils.createGetterAndSetter)(model, key, field.set, field.get); // if not array should be linked at _values directed

          model._values[key] = hydrated;
        }
      } // checkWith

    } else {
      // create getter and setter for that attribute inside _values
      (0, _utils.createGetterAndSetter)(model, key, field.set, field.get, field.checkHash); // just a value of the model

      if (data[key]) model._values[key] = data[key];
    }
  }

  return model;
};
/**
 * Check the wih_related if have permission at determined level
 *
 * @param {Integer} level
 * @param {String} nodeName
 * @param {Boolean} overwriteWith
 */


exports.hydrate = hydrate;

var checkWith = function checkWith(level, nodeName, with_related) {
  var ret = [false, true];
  with_related.forEach(function (item) {
    var _item$level;

    // found the attr named as model attribute
    if (((_item$level = item[level]) === null || _item$level === void 0 ? void 0 : _item$level.replace('!', '')) === nodeName) {
      ret = [true, !(item[level].indexOf('!') > -1)]; // [found_condition, isOptional]
    }
  });
  return ret;
};

exports.checkWith = checkWith;